<?php

declare(strict_types=1);

namespace Dedi\SyliusVoucherPlugin\Doctrine\ORM;

use Dedi\SyliusVoucherPlugin\Model\VoucherInterface;
use Dedi\SyliusVoucherPlugin\Repository\VoucherRepositoryInterface;
use Doctrine\ORM\QueryBuilder;
use Sylius\Bundle\ResourceBundle\Doctrine\ORM\EntityRepository;
use Sylius\Component\Channel\Model\ChannelInterface;
use Sylius\Component\Core\Model\CustomerInterface;
use Sylius\Component\Core\Model\OrderItemUnitInterface;

final class VoucherRepository extends EntityRepository implements VoucherRepositoryInterface
{
    public function createListQueryBuilder(): QueryBuilder
    {
        return $this->createQueryBuilder('o');
    }

    public function findOneEnabledByCodeAndChannel(string $code, ChannelInterface $channel): ?VoucherInterface
    {
        /** @var VoucherInterface|null $voucher */
        $voucher = $this->findOneBy([
            'code' => $code,
            'channel' => $channel,
            'enabled' => true,
        ]);

        return $voucher;
    }

    public function findOneByCode(string $code): ?VoucherInterface
    {
        /** @var VoucherInterface|null $voucher */
        $voucher = $this->findOneBy([
            'code' => $code,
        ]);

        return $voucher;
    }

    public function findOneByOrderItemUnit(OrderItemUnitInterface $orderItemUnit): ?VoucherInterface
    {
        /** @var VoucherInterface|null $voucher */
        $voucher = $this->findOneBy([
            'orderItemUnit' => $orderItemUnit,
        ]);

        return $voucher;
    }

    public function findEnabled(): array
    {
        return $this->findBy([
            'enabled' => true,
        ]);
    }

    public function createAccountListQueryBuilder(CustomerInterface $customer): QueryBuilder
    {
        $qb = $this->createListQueryBuilder();
        $qb->andWhere('o.customer = :customer');
        $qb->setParameter('customer', $customer);

        return $qb;
    }
}
