<?php

declare(strict_types=1);

namespace Dedi\SyliusVoucherPlugin\Form\Type;

use Dedi\SyliusVoucherPlugin\Controller\Action\AddVoucherToOrderCommand;
use Symfony\Component\Form\AbstractType;
use Symfony\Component\Form\DataTransformerInterface;
use Symfony\Component\Form\Extension\Core\Type\TextType;
use Symfony\Component\Form\FormBuilderInterface;
use Symfony\Component\OptionsResolver\OptionsResolver;

final class AddVoucherToOrderType extends AbstractType
{
    /** @var DataTransformerInterface */
    private $voucherToCodeDataTransformer;

    /** @var array */
    private $validationGroups;

    public function __construct(DataTransformerInterface $voucherToCodeDataTransformer, array $validationGroups)
    {
        $this->voucherToCodeDataTransformer = $voucherToCodeDataTransformer;
        $this->validationGroups = $validationGroups;
    }

    public function buildForm(FormBuilderInterface $builder, array $options): void
    {
        $builder
            ->add('voucher', TextType::class, [
                'label' => false,
                'attr' => [
                    'placeholder' => 'dedi_sylius_voucher.ui.enter_voucher_code',
                ],
                'invalid_message' => 'dedi_sylius_voucher.add_voucher_to_order_command.voucher.does_not_exist',
            ])
        ;

        $builder->get('voucher')->addModelTransformer($this->voucherToCodeDataTransformer);
    }

    public function configureOptions(OptionsResolver $resolver): void
    {
        $resolver->setDefaults([
            'data_class' => AddVoucherToOrderCommand::class,
            'validation_groups' => $this->validationGroups,
        ]);
    }

    public function getBlockPrefix(): string
    {
        return 'dedi_sylius_voucher_add_voucher_to_order';
    }
}
